@@ -19,6 +19,7 @@
 */
 
 #include "part.h"
+#include <kprotocolinfo.h>
 #include "event.h"
 #include "plugin.h"
 #include "mainwindow.h"
@@ -30,7 +31,6 @@
 #include <QtCore/QFileInfo>
 #include <QtGui/QPainter>
 #include <QtCore/QPoint>
-#include <QtCore/QTextStream>
 
 #include <kdirnotify.h>
 #include <kfiledialog.h>
@@ -146,7 +146,7 @@ void PartBase::setComponentData(const KComponentData &componentData, bool bLoadP
     KXMLGUIClient::setComponentData(componentData);
     KGlobal::locale()->insertCatalog(componentData.catalogName());
     // install 'instancename'data resource type
-    KGlobal::dirs()->addResourceType((componentData.componentName() + "data").toUtf8(),
+    KGlobal::dirs()->addResourceType(QString(componentData.componentName() + "data").toUtf8(),
                                      "data", componentData.componentName());
     if (bLoadPlugins) {
         loadPlugins(d->m_obj, this, componentData);
@@ -191,13 +191,13 @@ Part::~Part()
 {
     Q_D(Part);
 
-    //kDebug(1000) << "Part::~Part " << this;
+    //kDebug(1000) << this;
 
     if ( d->m_widget )
     {
         // We need to disconnect first, to avoid calling it !
-        disconnect( d->m_widget, SIGNAL( destroyed() ),
-                    this, SLOT( slotWidgetDestroyed() ) );
+        disconnect( d->m_widget, SIGNAL(destroyed()),
+                    this, SLOT(slotWidgetDestroyed()) );
     }
 
     if ( d->m_manager )
@@ -205,7 +205,7 @@ Part::~Part()
 
     if ( d->m_widget && d->m_autoDeleteWidget )
     {
-        kDebug(1000) << "deleting widget " << d->m_widget << " " << d->m_widget->objectName();
+        kDebug(1000) << "deleting widget" << d->m_widget << d->m_widget->objectName();
         delete static_cast<QWidget*>(d->m_widget);
     }
 
@@ -281,11 +281,9 @@ Part *Part::hitTest( QWidget *widget, const QPoint & )
 void Part::setWidget( QWidget *widget )
 {
     Q_D(Part);
-
-    assert ( !d->m_widget ); // otherwise we get two connects
     d->m_widget = widget;
-    connect( d->m_widget, SIGNAL( destroyed() ),
-             this, SLOT( slotWidgetDestroyed() ) );
+    connect( d->m_widget, SIGNAL(destroyed()),
+             this, SLOT(slotWidgetDestroyed()), Qt::UniqueConnection );
 }
 
 void Part::setSelectable( bool selectable )
@@ -351,8 +349,8 @@ void Part::slotWidgetDestroyed()
 
     d->m_widget = 0;
     if (d->m_autoDeletePart) {
-        kDebug(1000) << "KPart::slotWidgetDestroyed(), deleting part " << objectName();
-        delete this; // ouch, this should probably be deleteLater()
+        kDebug(1000) << "deleting part" << objectName();
+        this->deleteLater();
     }
 }
 
@@ -374,6 +372,7 @@ public:
     ReadOnlyPartPrivate(ReadOnlyPart *q): PartPrivate(q)
     {
         m_job = 0;
+        m_statJob = 0;
         m_uploadJob = 0;
         m_showProgressInfo = true;
         m_saveOk = false;
@@ -388,9 +387,13 @@ public:
     }
 
     void _k_slotJobFinished( KJob * job );
+    void _k_slotStatJobFinished(KJob * job);
     void _k_slotGotMimeType(KIO::Job *job, const QString &mime);
+    bool openLocalFile();
+    void openRemoteFile();
 
     KIO::FileCopyJob * m_job;
+    KIO::StatJob * m_statJob;
     KIO::FileCopyJob * m_uploadJob;
     KUrl m_originalURL; // for saveAs
     QString m_originalFilePath; // for saveAs
@@ -471,6 +474,7 @@ void ReadOnlyPart::setUrl(const KUrl &url)
     Q_D(ReadOnlyPart);
 
     d->m_url = url;
+    emit urlChanged( url );
 }
 
 QString ReadOnlyPart::localFilePath() const
@@ -487,19 +491,23 @@ void ReadOnlyPart::setLocalFilePath( const QString &localFilePath )
     d->m_file = localFilePath;
 }
 
+#ifndef KDE_NO_DEPRECATED
 bool ReadOnlyPart::isLocalFileTemporary() const
 {
     Q_D(const ReadOnlyPart);
 
     return d->m_bTemp;
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 void ReadOnlyPart::setLocalFileTemporary( bool temp )
 {
     Q_D(ReadOnlyPart);
 
     d->m_bTemp = temp;
 }
+#endif
 
 void ReadOnlyPart::setProgressInfoEnabled( bool show )
 {
@@ -538,67 +546,98 @@ bool ReadOnlyPart::openUrl( const KUrl &url )
     if ( !closeUrl() )
         return false;
     d->m_arguments = args;
-    d->m_url = url;
-    if ( d->m_url.isLocalFile() )
-    {
-        emit started( 0 );
-        d->m_file = d->m_url.toLocalFile();
-        d->m_bTemp = false;
-        // set the mimetype only if it was not already set (for example, by the host application)
-        if (d->m_arguments.mimeType().isEmpty())
-        {
-            // get the mimetype of the file
-            // using findByUrl() to avoid another string -> url conversion
-            KMimeType::Ptr mime = KMimeType::findByUrl(d->m_url, 0, true /* local file*/);
-            if (mime) {
-                d->m_arguments.setMimeType(mime->name());
-                d->m_bAutoDetectedMime = true;
-            }
-        }
-        bool ret = openFile();
-        if (ret) {
-            emit setWindowCaption( d->m_url.prettyUrl() );
-            emit completed();
-        } else emit canceled(QString());
-        return ret;
-    }
-    else
-    {
-        d->m_bTemp = true;
-        // Use same extension as remote file. This is important for mimetype-determination (e.g. koffice)
-        QString fileName = url.fileName();
-        QFileInfo fileInfo(fileName);
-        QString ext = fileInfo.completeSuffix();
-        QString extension;
-        if ( !ext.isEmpty() && url.query().isNull() ) // not if the URL has a query, e.g. cgi.pl?something
-            extension = '.'+ext; // keep the '.'
-        KTemporaryFile tempFile;
-        tempFile.setSuffix(extension);
-        tempFile.setAutoRemove(false);
-        tempFile.open();
-        d->m_file = tempFile.fileName();
+    setUrl(url);
 
-        KUrl destURL;
-        destURL.setPath( d->m_file );
+    d->m_file.clear();
+
+    if (d->m_url.isLocalFile()) {
+        d->m_file = d->m_url.toLocalFile();
+        return d->openLocalFile();
+    } else if (KProtocolInfo::protocolClass(url.protocol()) == ":local") {
+        // Maybe we can use a "local path", to avoid a temp copy?
         KIO::JobFlags flags = d->m_showProgressInfo ? KIO::DefaultFlags : KIO::HideProgressInfo;
-        flags |= KIO::Overwrite;
-        d->m_job = KIO::file_copy( d->m_url, destURL, 0600, flags );
-        d->m_job->ui()->setWindow( widget() ? widget()->topLevelWidget() : 0 );
-        emit started( d->m_job );
-        connect( d->m_job, SIGNAL( result( KJob * ) ), this, SLOT( _k_slotJobFinished ( KJob * ) ) );
-        connect(d->m_job, SIGNAL(mimetype(KIO::Job *, const QString &)),
-                this, SLOT(_k_slotGotMimeType(KIO::Job *, const QString&)));
+        d->m_statJob = KIO::mostLocalUrl(d->m_url, flags);
+        d->m_statJob->ui()->setWindow( widget() ? widget()->topLevelWidget() : 0 );
+        connect(d->m_statJob, SIGNAL(result(KJob*)), this, SLOT(_k_slotStatJobFinished(KJob*)));
+        return true;
+    } else {
+        d->openRemoteFile();
         return true;
     }
 }
 
+bool ReadOnlyPart::openFile()
+{
+    kWarning(1000) << "Default implementation of ReadOnlyPart::openFile called!"
+                   << metaObject()->className() << "should reimplement either openUrl or openFile.";
+    return false;
+}
+
+bool ReadOnlyPartPrivate::openLocalFile()
+{
+    Q_Q(ReadOnlyPart);
+    emit q->started( 0 );
+    m_bTemp = false;
+    // set the mimetype only if it was not already set (for example, by the host application)
+    if (m_arguments.mimeType().isEmpty()) {
+        // get the mimetype of the file
+        // using findByUrl() to avoid another string -> url conversion
+        KMimeType::Ptr mime = KMimeType::findByUrl(m_url, 0, true /* local file*/);
+        if (mime) {
+            m_arguments.setMimeType(mime->name());
+            m_bAutoDetectedMime = true;
+        }
+    }
+    const bool ret = q->openFile();
+    if (ret) {
+        emit q->setWindowCaption(m_url.prettyUrl());
+        emit q->completed();
+    } else {
+        emit q->canceled(QString());
+    }
+    return ret;
+}
+
+void ReadOnlyPartPrivate::openRemoteFile()
+{
+    Q_Q(ReadOnlyPart);
+    m_bTemp = true;
+    // Use same extension as remote file. This is important for mimetype-determination (e.g. koffice)
+    QString fileName = m_url.fileName();
+    QFileInfo fileInfo(fileName);
+    QString ext = fileInfo.completeSuffix();
+    QString extension;
+    if (!ext.isEmpty() && m_url.query().isNull()) // not if the URL has a query, e.g. cgi.pl?something
+        extension = '.'+ext; // keep the '.'
+    KTemporaryFile tempFile;
+    tempFile.setSuffix(extension);
+    tempFile.setAutoRemove(false);
+    tempFile.open();
+    m_file = tempFile.fileName();
+
+    KUrl destURL;
+    destURL.setPath( m_file );
+    KIO::JobFlags flags = m_showProgressInfo ? KIO::DefaultFlags : KIO::HideProgressInfo;
+    flags |= KIO::Overwrite;
+    m_job = KIO::file_copy(m_url, destURL, 0600, flags);
+    m_job->ui()->setWindow(q->widget() ? q->widget()->topLevelWidget() : 0);
+    emit q->started(m_job);
+    QObject::connect(m_job, SIGNAL(result(KJob*)), q, SLOT(_k_slotJobFinished(KJob*)));
+    QObject::connect(m_job, SIGNAL(mimetype(KIO::Job*,QString)),
+                     q, SLOT(_k_slotGotMimeType(KIO::Job*,QString)));
+}
+
 void ReadOnlyPart::abortLoad()
 {
     Q_D(ReadOnlyPart);
 
-    if ( d->m_job )
-    {
-        //kDebug(1000) << "Aborting job " << d->m_job;
+    if ( d->m_statJob ) {
+        //kDebug(1000) << "Aborting job" << d->m_statJob;
+        d->m_statJob->kill();
+        d->m_statJob = 0;
+    }
+    if ( d->m_job ) {
+        //kDebug(1000) << "Aborting job" << d->m_job;
         d->m_job->kill();
         d->m_job = 0;
     }
@@ -614,7 +653,7 @@ bool ReadOnlyPart::closeUrl()
 
     if ( d->m_bTemp )
     {
-        unlink( QFile::encodeName(d->m_file) );
+        QFile::remove( d->m_file );
         d->m_bTemp = false;
     }
     // It always succeeds for a read-only part,
@@ -623,11 +662,29 @@ bool ReadOnlyPart::closeUrl()
     return true;
 }
 
+void ReadOnlyPartPrivate::_k_slotStatJobFinished(KJob * job)
+{
+    Q_ASSERT(job == m_statJob);
+    m_statJob = 0;
+
+    // We could emit canceled on error, but we haven't even emitted started yet,
+    // this could maybe confuse some apps? So for now we'll just fallback to KIO::get
+    // and error again. Well, maybe this even helps with wrong stat results.
+    if (!job->error()) {
+        const KUrl localUrl = static_cast<KIO::StatJob*>(job)->mostLocalUrl();
+        if (localUrl.isLocalFile()) {
+            m_file = localUrl.toLocalFile();
+            (void)openLocalFile();
+            return;
+        }
+    }
+    openRemoteFile();
+}
+
 void ReadOnlyPartPrivate::_k_slotJobFinished( KJob * job )
 {
     Q_Q(ReadOnlyPart);
 
-    kDebug(1000) << "ReadOnlyPart::slotJobFinished";
     assert( job == m_job );
     m_job = 0;
     if (job->error())
@@ -643,8 +700,8 @@ void ReadOnlyPartPrivate::_k_slotJobFinished( KJob * job )
 
 void ReadOnlyPartPrivate::_k_slotGotMimeType(KIO::Job *job, const QString &mime)
 {
-    kDebug(1000) << "ReadOnlyPart::slotJobFinished:" << mime;
-    Q_ASSERT(job == m_job);
+    kDebug(1000) << mime;
+    Q_ASSERT(job == m_job); Q_UNUSED(job)
     // set the mimetype only if it was not already set (for example, by the host application)
     if (m_arguments.mimeType().isEmpty()) {
         m_arguments.setMimeType(mime);
@@ -660,7 +717,7 @@ void ReadOnlyPart::guiActivateEvent( GUIActivateEvent * event )
     {
         if (!d->m_url.isEmpty())
         {
-            kDebug(1000) << "ReadOnlyPart::guiActivateEvent -> " << d->m_url.prettyUrl();
+            kDebug(1000) << d->m_url;
             emit setWindowCaption( d->m_url.prettyUrl() );
         } else emit setWindowCaption( "" );
     }
@@ -674,7 +731,7 @@ bool ReadOnlyPart::openStream( const QString& mimeType, const KUrl& url )
     if ( !closeUrl() )
         return false;
     d->m_arguments = args;
-    d->m_url = url;
+    setUrl( url );
     return doOpenStream( mimeType );
 }
 
@@ -734,7 +791,7 @@ void ReadWritePart::setModified( bool modified )
 {
     Q_D(ReadWritePart);
 
-    kDebug(1000) << "ReadWritePart::setModified( " << (modified ? "true" : "false") << ")";
+    kDebug(1000) << "setModified(" << (modified ? "true" : "false") << ")";
     if ( !d->m_bReadWrite && modified )
     {
         kError(1000) << "Can't set a read-only document to 'modified' !" << endl;
@@ -842,6 +899,7 @@ bool ReadWritePart::saveAs( const KUrl & kurl )
     d->prepareSaving();
     bool result = save(); // Save local file and upload local file
     if (result) {
+        emit urlChanged( d->m_url );
         emit setWindowCaption( d->m_url.prettyUrl() );
     } else {
         d->m_url = d->m_originalURL;
@@ -862,10 +920,10 @@ void ReadWritePartPrivate::prepareSaving()
     {
         if ( m_bTemp ) // get rid of a possible temp file first
         {              // (happens if previous url was remote)
-            unlink( QFile::encodeName(m_file) );
+            QFile::remove( m_file );
             m_bTemp = false;
         }
-        m_file = m_url.path();
+        m_file = m_url.toLocalFile();
     }
     else
     { // Remote file
@@ -902,7 +960,7 @@ bool ReadWritePart::saveToUrl()
     {
         if (d->m_uploadJob)
         {
-            unlink(QFile::encodeName(d->m_uploadJob->srcUrl().path()));
+            QFile::remove(d->m_uploadJob->srcUrl().toLocalFile());
             d->m_uploadJob->kill();
             d->m_uploadJob = 0;
         }
@@ -920,7 +978,7 @@ bool ReadWritePart::saveToUrl()
         }
         d->m_uploadJob = KIO::file_move( uploadUrl, d->m_url, -1, KIO::Overwrite );
         d->m_uploadJob->ui()->setWindow( widget() ? widget()->topLevelWidget() : 0 );
-        connect( d->m_uploadJob, SIGNAL( result( KJob * ) ), this, SLOT( _k_slotUploadFinished (KJob *) ) );
+        connect( d->m_uploadJob, SIGNAL(result(KJob*)), this, SLOT(_k_slotUploadFinished(KJob*)) );
         return true;
     }
 }
@@ -931,11 +989,11 @@ void ReadWritePartPrivate::_k_slotUploadFinished( KJob * )
 
     if (m_uploadJob->error())
     {
-        unlink(QFile::encodeName(m_uploadJob->srcUrl().path()));
+        QFile::remove(m_uploadJob->srcUrl().toLocalFile());
         QString error = m_uploadJob->errorString();
         m_uploadJob = 0;
         if (m_duringSaveAs) {
-            m_url = m_originalURL;
+            q->setUrl(m_originalURL);
             m_file = m_originalFilePath;
         }
         emit q->canceled( error );
